"use strict";
const ts = require("typescript");
const path = require("path");
const babel = require("babel-core");
const fse = require("fs-extra");
const bluebird_1 = require("bluebird");
const basePath = process.cwd();
const tsConfigPath = path.join(basePath, "tsconfig.json");
const writeFile = bluebird_1.Promise.promisify(fse.writeFile);
const readFile = bluebird_1.Promise.promisify(fse.readFile);
const unlink = bluebird_1.Promise.promisify(fse.unlink);
main()
    .catch(error => {
    if (error instanceof CompilationError) {
        for (let diagnostic of error.errors) {
            const location = diagnostic.file.getLineAndCharacterOfPosition(diagnostic.start);
            const message = ts.flattenDiagnosticMessageText(diagnostic.messageText, '\n');
            console.log(`${diagnostic.file.fileName} (${location.line + 1},${location.character + 1}): ${message}`);
        }
    }
    else {
        console.error(error.stack || error.message || error);
    }
    process.exit(-1);
});
function main() {
    return __awaiter(this, void 0, void 0, function* () {
        const result = ts.parseConfigFileTextToJson(tsConfigPath, yield readText(tsConfigPath));
        if (result.error != null) {
            throw new CompilationError([result.error]);
        }
        result.config.noEmitOnError = false;
        return compile(result.config);
    });
}
function compile(config) {
    return __awaiter(this, void 0, void 0, function* () {
        const compilerOptions = ts.convertCompilerOptionsFromJson(config.compilerOptions, basePath);
        checkErrors(compilerOptions.errors);
        const program = ts.createProgram(config.files, compilerOptions.options);
        checkErrors(ts.getPreEmitDiagnostics(program));
        const outDir = compilerOptions.options.outDir;
        if (outDir == null) {
            throw new Error("outDir is not specified in the compilerOptions");
        }
        yield bluebird_1.Promise.promisify(fse.ensureDir)(outDir);
        const fileToSourceMap = {};
        const promises = [];
        const emittedFiles = new Set();
        program.emit(undefined, (fileName, data) => {
            emittedFiles.add(fileName);
            if (fileName.endsWith(".js")) {
                const sourceMapFileName = fileName + ".map";
                processCompiled(data, fileToSourceMap[sourceMapFileName], fileName, sourceMapFileName, promises);
            }
            else if (fileName.endsWith(".js.map")) {
                fileToSourceMap[fileName] = data;
            }
        });
        yield bluebird_1.Promise.all(promises);
        promises.length = 0;
        yield removedOld(outDir, emittedFiles, promises);
        return yield bluebird_1.Promise.all(promises);
    });
}
function removedOld(outDir, emittedFiles, promises) {
    return __awaiter(this, void 0, void 0, function* () {
        const files = yield bluebird_1.Promise.promisify(fse.readdir)(outDir);
        for (let file of files) {
            if (file[0] !== "." && !file.endsWith(".d.ts")) {
                const fullPath = outDir + '/' + file;
                if (!emittedFiles.has(fullPath)) {
                    promises.push(unlink(fullPath));
                }
            }
        }
    });
}
function processCompiled(code, sourceMap, jsFileName, sourceMapFileName, promises) {
    const result = babel.transform(code, {
        inputSourceMap: JSON.parse(sourceMap),
        sourceMaps: true,
        filename: jsFileName,
    });
    promises.push(writeFile(jsFileName, result.code), writeFile(sourceMapFileName, JSON.stringify(result.map)));
}
function checkErrors(errors) {
    if (errors.length !== 0) {
        throw new CompilationError(errors);
    }
}
function __awaiter(thisArg, _arguments, ignored, generator) {
    return bluebird_1.Promise.coroutine(generator).call(thisArg, _arguments);
}
function readText(file) {
    return readFile(file, "utf8");
}
class CompilationError extends Error {
    constructor(errors) {
        super("Compilation error");
        this.errors = errors;
    }
}
//# sourceMappingURL=builder.js.map